// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"


#include <memory>          // This gets the definition for auto_ptr
#include <string>
using namespace std;

//#include "Car.hpp"
// Pretend file "Car.hpp" contains the following (this is from FAQ 2.19):
class Car {
public:
  Car();             // Declare a constructor
  Car(int initRadioFreq, int horsepower); // Declare another constructor
  virtual void startEngine();
  virtual bool isRunning() const;
  virtual void tuneRadioTo(const string& band, int freq);
protected:
  bool isRunning_;
  bool radioOnAM_;
  int  radioFreq_;
  int  horsepower_;
};

typedef auto_ptr<Car> CarPtr;

void f()
{
  CarPtr p(new Car());       // 1: Create an object
  p->startEngine();          // 2: Call a member function
  p->tuneRadioTo("AM", 770); // 3: Call another member function
}                            // 4: Destroy the Car object

int main()
{
  f();
}

CarPtr g()
{
  CarPtr p(new Car());
  // ...
  return p;   // The caller is now responsible to delete the Car object
}

void h()
{
  CarPtr p = g();  // Ownership is transferred from g() to h() here
  // ...
}                  // The Car object dies here
