// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <memory>
using namespace std;

//#include "Car.hpp"
// Pretend file "Car.hpp" contains the following (this is from FAQ 2.17):

#include <string>
using namespace std;

class Car {
public:
	virtual void startEngine();     // Line 1
	virtual bool isRunning() const; // Line 2
	virtual void tuneRadioTo(const string& band, int freq); // Line 3
protected:
  bool isRunning_;                // Line 4
  bool radioOnAM_;
  int  radioFreq_;
};

void Car::startEngine()
{ isRunning_ = true; }

bool Car::isRunning() const
{ return isRunning_; }

void Car::tuneRadioTo(const string& band, int freq)
{ radioOnAM_ = (band == "AM"); radioFreq_ = freq; }

typedef auto_ptr<Car> CarPtr;

void f(Car& c)
{
  c.startEngine();
  // ...
}        //  The Car object is not deleted at this line

void g(CarPtr p)
{
  p->startEngine();
  // ...
}        //  The Car object is deleted at  this line

int main()
{
  CarPtr p(new Car());
  f(*p); // Pass-by-reference; *p can be used after this line
  g(p);  // Pass-by-auto_ptr; *p cannot be used after this line
}

//#include <memory>
//using namespace std;
//#include "Car.hpp"
//typedef auto_ptr<Car> CarPtr;

void h(Car* p)
{
  if (p == NULL) {
    // ...
  } else {
    p->startEngine();
    // ...
  }
}          //  Like pass-by-reference, the Car object is not deleted at this line

void i()
{
  h(NULL); // NULL is a valid parameter to function h()

  CarPtr p(new Car());
  h(p.get()); // Pass-by-pointer; *p can be used after this line
  // ...
}          //  The Car object is deleted at  this line
