// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <iostream>
using namespace std;

class Shape {
public:
  Shape(int x, int y)             throw();
  virtual ~Shape()                throw();
  virtual void draw() const       throw() = 0;
  virtual void move(int x, int y) throw() = 0;
protected:
  int x_, y_;
  void operator= (const Shape& s) throw();
  Shape(const Shape& s) throw();
};

Shape::Shape(int x, int y) throw()
: x_(x)
, y_(y)
{ }

Shape::Shape(const Shape& s) throw()
: x_(s.x_)
, y_(s.y_)
{ }

Shape::~Shape() throw()
{ }

void Shape::operator= (const Shape& s) throw()
{
  x_ = s.x_;
  y_ = s.y_;
}

void dragAndDrop(Shape& s) throw()  // A polymorphic function
{
  s.move(42,24);   // Dynamic binding calls the "right" code
  s.draw();        // (as if the compiler predicted the future)
}

class Square : public Shape {
public:
  Square(int x, int y, int width) throw();
  virtual void draw() const       throw();
  virtual void move(int x, int y) throw();
protected:
  int width_;
};

Square::Square(int x, int y, int width) throw()
: Shape(x,y)
, width_(width)
{ }

void Square::draw() const throw()
{ cout << "Square::draw\n"; }

void Square::move(int x, int y) throw()
{
  x_ = x;
  y_ = y;
  cout << "Square::move\n";
}

class Circle : public Shape {
public:
  Circle(int x, int y, int radius) throw();
  virtual void draw() const        throw();
  virtual void move(int x, int y)  throw();
protected:
  int radius_;
};

Circle::Circle(int x, int y, int radius) throw()
: Shape(x,y)
, radius_(radius)
{ }

void Circle::draw() const throw()
{ cout << "Circle::draw\n"; }

void Circle::move(int x, int y) throw()
{
  x_ = x;
  y_ = y;
  cout << "Circle::move\n";
}

int main()
{
  Square s = Square(5, 20, 3);
  Circle c = Circle(10, 15, 7);
  dragAndDrop(s);
  dragAndDrop(c);
}
