// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <iostream>
#include <string>
using namespace std;

class Matrix {
public:
  Matrix() throw();
  friend ostream& operator<<(ostream&, Matrix&);
  // ...
};

#include <stdexcept>

Matrix add(const Matrix& a, const Matrix& b)
             throw(overflow_error,underflow_error);
Matrix sub(const Matrix& a, const Matrix& b)
             throw(overflow_error,underflow_error);
Matrix mul(const Matrix& a, const Matrix& b)
             throw(overflow_error,underflow_error);
Matrix div(const Matrix& a, const Matrix& b)
             throw(overflow_error,underflow_error);

void solutionA(const Matrix& a, const Matrix& b)
                 throw(underflow_error)
{
  try {
    cout << "a + b is " << add(a, b) << '\n';
    cout << "a - b is " << sub(a, b) << '\n';
    cout << "a * b is " << mul(a, b) << '\n';
    cout << "a / b is " << div(a, b) << '\n';
  }
  catch (overflow_error& e) {
    cout << "overflow: " << e.what() << '\n';
  }
}

enum ReturnCode {
  OK,
  OVERFLOW_ERROR,
  UNDERFLOW_ERROR
};

ReturnCode add(Matrix& result, const Matrix& a, const Matrix& b);
ReturnCode sub(Matrix& result, const Matrix& a, const Matrix& b);
ReturnCode mul(Matrix& result, const Matrix& a, const Matrix& b);
ReturnCode div(Matrix& result, const Matrix& a, const Matrix& b);

ReturnCode solutionB(const Matrix& a, const Matrix& b)
{
  Matrix result;
  ReturnCode rc;

  rc = add(result, a, b);
  if (rc == OK) {
    cout << "a + b is " << result << '\n';
  } else if (rc == OVERFLOW_ERROR) {
    cout << "overflow error: Matrix + Matrix\n";
    return OK;                  // Overflow has been handled so return normally
  } else {
    return rc;                  // Some other error such as Underflow
  }

  rc = sub(result, a, b);
  if (rc == OK) {
    cout << "a - b is " << result << '\n';
  } else if (rc == OVERFLOW_ERROR) {
    cout << "overflow error: Matrix - Matrix\n";
    return OK;
  } else {
    return rc;
  }

  rc = mul(result, a, b);
  if (rc == OK) {
    cout << "a * b is " << result << '\n';
  } else if (rc == OVERFLOW_ERROR) {
    cout << "overflow error: Matrix * Matrix\n";
    return OK;
  } else {
    return rc;
  }

  rc = div(result, a, b);
  if (rc == OK) {
    cout << "a / b is " << result << '\n';
  } else if (rc == OVERFLOW_ERROR) {
    cout << "overflow error: Matrix / Matrix\n";
    return OK;
  } else {
    return rc;
  }

  return OK;
}
