// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <new>
#include <stdexcept>
#include <iostream>
using namespace std;

class MyString {
public:
  MyString(const char* s) throw(bad_alloc);
  ~MyString() throw();
  MyString(const MyString& s) throw(bad_alloc);
  MyString& operator= (const MyString& s) throw(bad_alloc);
  unsigned size() const throw();
  char& operator[] (unsigned index)       throw(out_of_range);
  char  operator[] (unsigned index) const throw(out_of_range);
  void toUpper() throw();  //capitalizes (*this)[i] for valid i
protected:
  unsigned len_;  //ORDER DEPENDENCY
  char* data_;    //ORDER DEPENDENCY
};

MyString::MyString(const char* s) throw(bad_alloc)
: len_(strlen(s)), data_(new char[len_+1]) { strcpy(data_,s); }
MyString::MyString(const MyString& s) throw(bad_alloc)
: len_(s.len_), data_(new char[len_+1]) { strcpy(data_,s.data_); }
MyString::~MyString() throw()
{ delete [] data_; }
unsigned MyString::size() const throw()
{ return len_; }
char& MyString::operator[] (unsigned index)       throw(out_of_range)
{ if (index >= len_) throw out_of_range("index >= len_"); return data_[index]; }
char  MyString::operator[] (unsigned index) const throw(out_of_range)
{ if (index >= len_) throw out_of_range("index >= len_"); return data_[index]; }
void MyString::toUpper() throw()
{ _strupr(data_); }

int main()
{
  MyString s = "xyz";
  for (unsigned i = 0; i < s.size(); ++i)
    cout << "Character #" << i << " is " << s[i] << '\n';
  s.toUpper();
}
