// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

class Base;

class Fred {
  friend Base;
};

class Base {
  //member functions of Base are friends of Fred
};

class Derived : public Base {
  //member functions of Derived are not friends of Fred
};

class EggCartonFiller;   // tell the compiler that "EggCartonFiller" is a class

class EggCarton {
public:
  EggCarton() throw();   // creates an empty carton
private:
  friend EggCartonFiller;
  int numEggs_;          // numEggs_ can't exceed a dozen
};

EggCarton::EggCarton()
: numEggs_(0) { }

class EggCartonFiller {
public:
  void addAnEgg(EggCarton& carton) throw();
};

void EggCartonFiller::addAnEgg(EggCarton& carton) throw()
{
  if (carton.numEggs_ < 12)
    ++ carton.numEggs_;
}

class SubversiveFiller : public EggCartonFiller {
public:
  void violateEncapsulation(EggCarton& carton) throw();
};

void SubversiveFiller::violateEncapsulation(EggCarton& carton) throw()
{
  #ifdef GENERATE_ERROR
    carton.numEggs_ = 13;  // Compile-time error: can't access carton.numEggs_
  #endif
}
