// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <iostream>
using namespace std;

class Shape {
public:
  virtual ~Shape() throw();
  friend ostream& operator<< (ostream& ostr, const Shape& s) throw();
protected:
  virtual void print(ostream& ostr) const throw() = 0;
};

inline ostream& operator<< (ostream& ostr, const Shape& s) throw()
{
  s.print(ostr);  // The friend calls a protected: virtual
  return ostr;
}

Shape::~Shape() throw()
{ }

class Circle : public Shape {
public:
  Circle() throw();
protected:
  virtual void print(ostream& ostr) const throw();
    //        Derived classes override the member, not the friend
  float radius_;
};

Circle::Circle() throw()
: radius_(42) { }

void Circle::print(ostream& ostr) const throw()
{ ostr << "Circle of radius " << radius_; }

void userCode(Shape& s)
{ cout << s << '\n'; }  // Users use the friend, not the member

int main()
{
  Circle c;
  userCode(c);
}
