// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <new>
#include <cstdlib>
using namespace std;

class List {
public:
  List()                          throw();
  List(const List& a)             throw(bad_alloc);
 ~List()                          throw();
  List& operator= (const List& a) throw(bad_alloc);
  void prepend(int e)             throw(bad_alloc);
  bool isEmpty() const            throw();
  void clear()                    throw();
private:
  class Node;    // The private class is called List::Node
  Node* first_;
};

class List::Node {
private:         // List::Node has no public: members
  friend List;   // List::Node declares List as its friend
  Node(int e, Node* next=NULL) throw();
  Node* next_;
  int   elem_;
};

List::Node::Node(int e, Node* next) throw() : next_(next), elem_(e) { }
List::List()               throw()          : first_(NULL) { }
List::List(const List& a)  throw(bad_alloc) : first_(NULL) { *this = a; }
List::~List()              throw()          { clear(); }
void List::prepend(int e)  throw(bad_alloc) { first_ = new Node(e, first_); }
bool List::isEmpty() const throw()          { return first_ == NULL; }
List& List::operator= (const List& a) throw(bad_alloc)
{   if (this == &a) return *this;
	clear();
	Node* last = NULL;
	for (Node* cur=a.first_; cur!=NULL; cur=cur->next_) {
		if (cur==a.first_) {
			first_ = last = new Node(cur->elem_);
		}
		else {
			last->next_ = new Node(cur->elem_);
			last = last->next_;
		}
	}
	return *this; 
}
void List::clear() throw()
{   while (first_) {
		Node* tmpFirst = first_;
		first_ = first_->next_;
		delete tmpFirst;
	}
}

int main()
{
  List a;
  a.prepend(4);
  a.prepend(3);
  a.prepend(2);
  List b;
  b = a;
}
