// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <iostream>
using namespace std;

void lock(int i) throw()
{
  cout << "pretend we acquire lock #" << i << '\n';
  //in reality, this would manipulate semaphore #i
  // (or use some other mutual exclusion primitive).
}

void unlock(int i) throw()
{
  cout << "pretend we release lock #" << i << '\n';
  //in reality, this would manipulate semaphore #i
  // (or use some other mutual exclusion primitive).
}

class Lock {
public:
  Lock(int lockNum) throw();
 ~Lock()            throw();
protected:
  int lockNum_;
private:
  // These are never defined (copying a Lock is senseless)
  Lock(const Lock&);
  Lock& operator= (const Lock&);
};

inline Lock::Lock(int lockNum) throw()
: lockNum_(lockNum) { lock(lockNum_); }

inline Lock::~Lock() throw()
{ unlock(lockNum_); }

void multiThreadedFunction() throw()
{
  cout << "no mutual exclusion here\n";
  {
    Lock lock(42);   //pretend this is critical section #42
    cout << "lock provides mutual exclusion here\n";
  }

  cout << "no mutual exclusion here\n";
}
