// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <new>
#include <memory>
#include <iostream>
using namespace std;

class Shape;
typedef auto_ptr<Shape>  ShapePtr;

class Shape {
public:
           Shape()                       throw();
  virtual ~Shape()                       throw();
  virtual void draw() const              throw() = 0;
  virtual ShapePtr clone() const         throw(bad_alloc) = 0;
  virtual ShapePtr createSimilar() const throw(bad_alloc) = 0;
};

Shape::Shape()  throw() { }
Shape::~Shape() throw() { }

class Circle : public Shape {
public:
  Circle(int radius=0)                   throw();
  virtual void draw() const              throw();
  virtual ShapePtr clone() const         throw(bad_alloc);
  virtual ShapePtr createSimilar() const throw(bad_alloc);
protected:
  int radius_;
};

Circle::Circle(int radius) throw()
: Shape(), radius_(radius) { }

void Circle::draw() const throw()
{ cout << "Circle: radius=" << radius_ << '\n'; }

ShapePtr Circle::createSimilar() const throw(bad_alloc)
{ return new Circle(); }

ShapePtr Circle::clone() const throw(bad_alloc)
{ return new Circle(*this); }

void userCode(ShapePtr s) throw()
{
  cout << "userCode() number 1: ";
  s->draw();

  ShapePtr copy = s->clone();
  cout << "userCode() number 2: ";
  copy->draw();

  ShapePtr similar = s->createSimilar();
  cout << "userCode() number 3: ";
  similar->draw();
}      // copy and similar will be automatically deleted here; see FAQ 2.07

int main()
{
  ShapePtr c(new Circle(42));
  cout << "main() number 1: ";
  c->draw();
  userCode(c);
       // Because of auto_ptr's copy semantics, c will be NULL here; see FAQ 2.11
}
