// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

//#include "Array.hpp"
// Pretend "Array.hpp" contains the following (this is from FAQ 25.02):
#include <stdexcept>
using namespace std;

template<class T>    // T is a type in the declaration that follows
class Array {
public:
  Array(unsigned size=10);
  Array(const Array<T>& a);                //copy constructor
  Array<T>& operator= (const Array<T>& a); //assignment
 ~Array() throw();
  unsigned size() const throw();
  const T& operator[] (unsigned i) const throw(out_of_range);
  T&       operator[] (unsigned i)       throw(out_of_range);
protected:
  unsigned  size_;
  T*        arr_;
};

template<class T> inline Array<T>::Array(unsigned size)
: size_(size)
, arr_(new T[size])
{ }

template<class T> inline Array<T>::~Array() throw()
{ delete[] arr_;  }

template<class T> inline unsigned Array<T>::size() const throw()
{ return size_; }

template<class T> inline const T& Array<T>::operator[] (unsigned i) const
throw(out_of_range)
{
  if (i >= size_) throw out_of_range("Array<T>::operator[]"));
  return arr_[i];
}

template<class T> inline T& Array<T>::operator[] (unsigned i)
throw(out_of_range)
{
  if (i >= size_) throw out_of_range("Array<T>::operator[] const");
  return arr_[i];
}

#include <new>
#include <memory.h>
using namespace std;

template<class T> class Array;

template<>
class Array<bool> {
public:
  typedef  unsigned char  Byte;

  class BitRef {
  public:
    BitRef(Byte& byte, unsigned bit) throw();
    operator bool() throw();
    BitRef& operator= (bool b) throw();
  private:
    Byte& byte_;
    unsigned bit_;
  };

  Array(unsigned size=10) throw(bad_alloc);
  Array(const Array<bool>& a) throw(bad_alloc);
  Array<bool>& operator= (const Array<bool>& a) throw(bad_alloc);
  ~Array() throw();
  unsigned size() const throw();
  bool   operator[] (unsigned i) const throw();
  BitRef operator[] (unsigned i)       throw();
protected:
  unsigned size_;
  Byte*    arr_;
};

template<> inline Array<bool>::Array(unsigned size) throw(bad_alloc)
: size_(size)
, arr_ (new Byte[(size+7u)/8u])
{ memset(arr_, '\0', (size+7u)/8u); }

template<> inline Array<bool>::~Array() throw()
{ delete[] arr_; }

template<> inline unsigned Array<bool>::size() const throw()
{ return size_; }

template<> inline bool Array<bool>::operator[] (unsigned i) const
throw()
{ return (arr_[i/8u] >> (i&7u)) & 1u; }

template<> inline Array<bool>::BitRef Array<bool>::operator[] (unsigned i)
throw()
{ return BitRef(arr_[i/8u], i&7u); }

template<> inline Array<bool>::BitRef::BitRef(Byte& byte, unsigned bit) throw()
: byte_(byte)
, bit_(bit)
{ }

template<> inline Array<bool>::BitRef::operator bool() throw()
{ return (byte_ >> bit_) & 1u; }

template<> inline Array<bool>::BitRef& Array<bool>::BitRef::operator= (bool b) throw()
{
  if (b) byte_ |=  (1u << bit_);
  else   byte_ &= ~(1u << bit_);
  return *this;
}

int main()
{
  Array<bool> ab;
  ab[5] = true;
}

