// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <stdexcept>
#include <iostream>
#include <string>
using namespace std;

class BadFileName : public runtime_error {
public:
  BadFileName(const string& filename) throw();
};

BadFileName::BadFileName(const string& filename) throw()
: runtime_error("bad file name: " + filename) { }

class AccessViolation : public runtime_error {
public:
  AccessViolation(const string& filename) throw();
};

AccessViolation::AccessViolation(const string& filename) throw()
: runtime_error("access violation on file " + filename) { }

class File {
public:
  File(const string& filename) throw(BadFileName, AccessViolation);
};

File::File(const string& filename) throw(BadFileName, AccessViolation)
{
  cout << "File::File(): Opening " << filename << "\n";
  if (filename == "NoAccess.txt")
    throw AccessViolation(filename);
  if (filename == "BadName.txt")
    throw BadFileName(filename);
  cout << "  Successfully opened\n";
}

void first(const string& filename) throw();
void second(const string& filename) throw(BadFileName);

int main()
{
  first("GoodFile.txt");
  first("BadName.txt");
  first("NoAccess.txt");
}

void first(const string& filename) throw()
{
  try {
    second(filename);
  }
  catch (BadFileName& e) {
    cout << "  first(): " << e.what() << ": Finish recovery\n";
  }
}

void second(const string& filename) throw(BadFileName)
{
  try {
    File x(filename);
  }
  catch (BadFileName& e) {
    cout << "  second(): " << e.what() << ": Partial recovery\n";
    throw;   // Re-throw the BadFileName exception
  }
  catch (AccessViolation& e) {
    cout << "  second(): " << e.what() << ": Full recovery\n";
  }
}
