// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <cstring>
#include <stdexcept>
using namespace std;

class EvilString {
public:
  EvilString(const char* s) throw(bad_alloc);
 ~EvilString() throw();
              // Since this contains remote ownership, it needs an explicit copy constructor
              // But pretend the developer failed to provide an explicit copy constructor
              // Similar comments for the assignment operator
protected:
  unsigned len_;     // ORDER DEPENDENCY; see FAQ 22.10
  char* data_;       // ORDER DEPENDENCY; see FAQ 22.10
};

EvilString::EvilString(const char* s) throw(bad_alloc)
: len_(strlen(s))
, data_(new char[len_ + 1])
{ memcpy(data_, s, len_ + 1); }

EvilString::~EvilString() throw()
{ delete[] data_; }

void sample(EvilString b) throw(bad_alloc)
{
  // Since EvilString lacks a proper copy constructor,
  // changes to b's string-data will also change a's string
}

int main()
{
  EvilString a = "xyzzy";
  sample(a);
  // Any use of a might corrupt the heap
}
