// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

class FredPtr;

class Fred {
public:
  Fred() throw() : count_(0) /*...*/ { }  // All ctors set count_ to 0 !
  // ...
private:
  friend FredPtr;     // A friend class; see FAQ 19.01
  unsigned count_;
  // count_ must be initialized to 0 by all constructors
  // count_ is the number of FredPtr objects that point at this
};

class FredPtr {
public:
  FredPtr(Fred* p) throw()    : p_(p) { if (p_) ++p_->count_; }
 ~FredPtr() throw()           { if (p_ && --p_->count_ == 0) delete p_; }
  FredPtr(const FredPtr& p) throw() : p_(p.p_) { if (p_) ++p_->count_; }
  FredPtr& operator= (const FredPtr& p) throw();
  Fred* operator-> () throw() { return p_;  }
  Fred& operator* () throw()  { return *p_; }
  Fred* getRaw() throw()      { return p_;  }
private:
  Fred* p_;
};

FredPtr& FredPtr::operator= (const FredPtr& p) throw()
{
  // DO NOT CHANGE THE ORDER OF THESE STATEMENTS!
  // (This order properly handles self-assignment
  if (p.p_) ++p.p_->count_;
  if (p_ && --p_->count_ == 0) delete p_;
  p_ = p.p_;
  return *this;
}
